package thehippomaster.animationapi.client.model.wavefront;

import net.minecraft.client.renderer.Tessellator;

public class Vertex
{
  public Vertex(float x, float y, float z)
  {
    this.x = x;
    this.y = y;
    this.z = z;
  }
  
  public Vertex(double x, double y, double z)
  {
    this((float)x, (float)y, (float)z);
  }
  
  public Vertex smooth()
  {
    float sq = (float)Math.sqrt(this.x * this.x + this.y * this.y + this.z * this.z);
    this.x /= sq;
    this.y /= sq;
    this.z /= sq;
    return this;
  }
  
  public Vertex tessellate(Tessellator tessellator)
  {
    return tessellateWithUV(tessellator, null);
  }
  
  public Vertex tessellateWithUV(Tessellator tessellator, UV uv)
  {
    if (uv == null) {
      Tessellator.func_178181_a().func_178180_c().func_181662_b(this.x, this.y, this.z);
    } else {
      Tessellator.func_178181_a().func_178180_c().func_181662_b(this.x, this.y, this.z).func_187315_a(uv.getU(), uv.getV());
    }
    return this;
  }
  
  public Vertex add(double x, double y, double z)
  {
    return new Vertex(this.x + x, this.y + y, this.z + z);
  }
  
  public Vertex add(Vertex v)
  {
    return add(v.x, v.y, v.z);
  }
  
  public Vertex mul(double c)
  {
    return new Vertex(c * this.x, c * this.y, c * this.z);
  }
  
  public String toString()
  {
    return String.format("Vertex(%s, %s, %s)", new Object[] { Float.valueOf(this.x), Float.valueOf(this.y), Float.valueOf(this.z) });
  }
  
  public static Vertex unitX = new Vertex(1, 0, 0);
  public static Vertex unitY = new Vertex(0, 1, 0);
  public static Vertex unitZ = new Vertex(0, 0, 1);
  public static Vertex unitNX = new Vertex(-1, 0, 0);
  public static Vertex unitNY = new Vertex(0, -1, 0);
  public static Vertex unitNZ = new Vertex(0, 0, -1);
  public static Vertex unitPYNZ = new Vertex(0.0D, 0.707D, -0.707D);
  public static Vertex unitPXPY = new Vertex(0.707D, 0.707D, 0.0D);
  public static Vertex unitPYPZ = new Vertex(0.0D, 0.707D, 0.707D);
  public static Vertex unitNXPY = new Vertex(-0.707D, 0.707D, 0.0D);
  public float x;
  public float y;
  public float z;
}
