package thehippomaster.animationapi.client.model.wavefront;

import net.minecraft.util.ResourceLocation;
import thehippomaster.animationapi.util.Color;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Hashtable;

public class WavefrontModel
{
  private String mtlName;
  private String modid;
  private String directory;
  public ArrayList<Vertex> vertex;
  public ArrayList<UV> uv;
  public Hashtable<String, String> nameToStringHash;
  public Hashtable<String, Part> nameToPartHash;
  public float xDim;
  public float yDim;
  public float zDim;
  public float xMin;
  public float yMin;
  public float zMin;
  public float xMax;
  public float yMax;
  public float zMax;
  public float dimMax;
  public float dimMaxInv;
  
  public WavefrontModel()
  {
    this.vertex = new ArrayList();
    this.uv = new ArrayList();
    this.nameToStringHash = new Hashtable();
    this.nameToPartHash = new Hashtable();
  }
  
  public ResourceLocation getAlternativeTexture(String name)
  {
    ResourceLocation resource = new ResourceLocation(this.modid, this.directory.substring(1) + name);
    return resource;
  }
  
  public boolean load(String path)
  {
    int lastSlashId = path.lastIndexOf('/');
    this.modid = "animationapi";
    this.directory = path.substring(0, lastSlashId + 1);
    Part part = null;
    FaceGroup fg = null;
    try
    {
      loadOBJ(path + ".obj", part, fg);
      part = null;
      loadMTL(path + ".mtl");
      part = null;
    }
    catch (Exception e)
    {
      return false;
    }
    this.xDim = (this.xMax - this.xMin);
    this.yDim = (this.yMax - this.yMin);
    this.zDim = (this.zMax - this.zMin);
    this.dimMax = Math.max(Math.max(this.xMax, this.yMax), this.zMax);
    this.dimMaxInv = (1.0F / this.dimMax);
    return true;
  }
  
  private boolean loadOBJ(String path, Part part, FaceGroup fg)
    throws Exception
  {
    InputStream stream = WavefrontModel.class.getResourceAsStream(path);
    BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(stream, "UTF-8"));
    String line;
    while ((line = bufferedReader.readLine()) != null)
    {
      String[] words = line.split(" ");
      String command = words[0];
      if (command.equals("o"))
      {
        part = new Part(words[1], this.vertex, this.uv);
        this.nameToPartHash.put(words[1], part);
      }
      else if (command.equals("v"))
      {
        Vertex v = new Vertex(Float.parseFloat(words[1]), Float.parseFloat(words[2]), Float.parseFloat(words[3]));
        part.addVertex(v);
        this.xMin = Math.min(this.xMin, v.x);
        this.yMin = Math.min(this.yMin, v.y);
        this.zMin = Math.min(this.zMin, v.z);
        this.xMax = Math.max(this.xMax, v.x);
        this.yMax = Math.max(this.yMax, v.y);
        this.zMax = Math.max(this.zMax, v.z);
      }
      else if (command.equals("vt"))
      {
        part.getUV().add(new UV(Float.parseFloat(words[1]), 1.0F - Float.parseFloat(words[2])));
      }
      else if (command.equals("f"))
      {
        int vertexNbr = words.length - 1;
        if (vertexNbr == 3)
        {
          Vertex[] verticeId = new Vertex[vertexNbr];
          UV[] uvId = new UV[vertexNbr];
          for (int idx = 0; idx < vertexNbr; idx++)
          {
            String[] id = words[(idx + 1)].split("/");
            verticeId[idx] = ((Vertex)part.getVertices().get(Integer.parseInt(id[0]) - 1));
            if ((id.length > 1) && (!id[1].equals(""))) {
              uvId[idx] = ((UV)part.getUV().get(Integer.parseInt(id[1]) - 1));
            } else {
              uvId[idx] = null;
            }
          }
          fg.faces.add(new Face(verticeId, uvId, new Normal(verticeId[0], verticeId[1], verticeId[2])));
        }
      }
      else if (command.equals("mtllib"))
      {
        this.mtlName = words[1];
      }
      else if (command.equals("usemtl"))
      {
        fg = new FaceGroup();
        fg.mtlName = words[1];
        if ((part != null) && (part.faceGroup != null)) {
          part.faceGroup.add(fg);
        }
      }
    }
    bufferedReader.close();
    return true;
  }
  
  private boolean loadMTL(String path)
    throws Exception
  {
    InputStream stream = WavefrontModel.class.getResourceAsStream(path);
    if ((stream == null) || (path == null))
    {
      stream.close();
      return false;
    }
    BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(stream, "UTF-8"));
    String mtlName = "";
    String line;
    String[] words;
    while ((line = bufferedReader.readLine()) != null)
    {
      words = line.split(" ");
      String command = words[0];
      if (command.equals("newmtl")) {
        mtlName = words[1];
      } else if (command.equals("map_Kd")) {
        for (Part partPtr : this.nameToPartHash.values()) {
          for (FaceGroup faceGroup : partPtr.faceGroup) {
            if ((faceGroup.mtlName != null) && (faceGroup.mtlName.equals(mtlName))) {
              if (!words[1].contains(this.modid)) {
                faceGroup.resource = new ResourceLocation(this.modid, "models/" + this.directory + words[1]);
              } else {
                faceGroup.resource = new ResourceLocation(words[1] + ".png");
              }
            }
          }
        }
      } else if (command.equals("Kd")) {
        for (Part partPtr : this.nameToPartHash.values()) {
          for (FaceGroup faceGroup : partPtr.faceGroup) {
            if ((faceGroup.mtlName != null) && (faceGroup.mtlName.equals(mtlName))) {
              faceGroup.color = new Color(Float.parseFloat(words[1]), Float.parseFloat(words[2]), Float.parseFloat(words[3]), 1.0F);
            }
          }
        }
      }
    }
    bufferedReader.close();
    return true;
  }
  
  public void draw(String part)
  {
    Part partPtr = getPart(part);
    if (partPtr != null) {
      partPtr.draw();
    }
  }
  
  public Part getPart(String part)
  {
    return (Part)this.nameToPartHash.get(part);
  }
  
  public String getString(String name)
  {
    return (String)this.nameToStringHash.get(name);
  }
  
  public String getMtlName()
  {
    return this.mtlName;
  }
}
